forecast_rGARCH_GAS_n <- function(est, fitted, steps = 1, type = "", simulations = 100) {
	σ <- est$optim_vals[["σ"]]
	φ <- est$optim_vals[["φ"]]
	d1 <- est$optim_vals[["d1"]]
	d2 <- est$optim_vals[["d2"]]
	λ <- fitted[["lambda"]][length(fitted[["lambda"]])]
	# ψ <- est$optim_vals[["ψ"]]
	α <- est$optim_vals[["α"]]
	β <- est$optim_vals[["β"]]
	ω <- est$optim_vals[["ω"]]

	# Simulation approach
	if (type == "simulation") {
		nsims <- simulations
		# Generate the errors
		sim_u <- rnorm(nsims)
		sim_z <- rnorm(nsims)

		# Prealocate the matrix for predictions
		pred <- matrix(rep(NA, steps*nsims), nrow = nsims)

		# Predict for every simulated tuple of residuals
		for (j in 1:nsims) {
			z <- sim_z[i]
			u <- sim_u[i]

			Δ = 0.5 * (z^2 - 1) + u/σ * (φ - d1*z^2 - d2/2 * z)
			S = ((1/σ)^2 * (φ^2 + 3*d1^2 + d2^2 / 4 - 2 * d1 * φ) + 0.5)^(-1)

			# pred[j,1] <- ω + α*Δ*S + β*λ + ψ*z
			pred[j,1] <- ω + α*Δ*S + β*λ

			if (steps == 1) {
				return(pred)
			}

			for (i in 1:(steps-1)) {
				# Because ε is martingale difference sequence the expected value should be zero	
				pred[j,i+1] <- ω + β * pred[i]
			}
		}
		# Escape
		return(pred)
	}

	# Bootstrap approach
	if (type == "simulation") {
		nboots <- simulations
		# Which errors to take from the fitted, with replacement!
		indeces <- sample(1:length(fitted[["lambda"]]), nboots, replace = T)

		boot_u <- fitted[["u"]][indeces]
		boot_z <- fitted[["z"]][indeces]

		pred <- matrix(rep(NA, steps*nboots), nrow = nboots)

		# Predict for every simulated tuple of residuals
		for (j in 1:nboots) {
			z <- boot_z[i]
			u <- boot_u[i]

			Δ = 0.5 * (z^2 - 1) + u/σ * (φ - d1*z^2 - d2/2 * z)
			S = ((1/σ)^2 * (φ^2 + 3*d1^2 + d2^2 / 4 - 2 * d1 * φ) + 0.5)^(-1)

			# pred[j,1] <- ω + α*Δ*S + β*λ + ψ*z
			pred[j,1] <- ω + α*Δ*S + β*λ

			if (steps == 1) {
				return(pred)
			}

			for (i in 1:(steps-1)) {
				# Because ε is martingale difference sequence the expected value should be zero	
				pred[j,i+1] <- ω + β * pred[i]
			}
		}
		# Escape
		return(pred)
	}

	# If no type set, defaulting to the standard forecasting.
	z <- fitted[["z"]][length(fitted[["z"]])]
	u <- fitted[["u"]][length(fitted[["u"]])]

	Δ = 0.5 * (z^2 - 1) + u/σ * (φ - d1*z^2 - d2/2 * z)
	S = ((1/σ)^2 * (φ^2 + 3*d1^2 + d2^2 / 4 - 2 * d1 * φ) + 0.5)^(-1)

	if (length(steps) == 1) {
		pred <- rep(NA, steps)

		# pred[1] <- ω + α*Δ*S + β*λ + ψ*z
		pred[1] <- ω + α*Δ*S + β*λ

		if (steps == 1) {
			return(pred)
		}

		for (i in 1:(steps-1)) {
			# Because ε is martingale difference sequence the expected value should be zero	
			pred[i+1] <- ω + β * pred[i]
		}
		return(pred)
	} else {
		output <- list()
		for (i in 1:length(steps)) {
			
			pred <- rep(NA, times = steps[i])
			
			# pred[1] <- ω + α*Δ*S + β*λ + ψ*z
			pred[1] <- ω + α*Δ*S + β*λ

			if (steps[i] == 1) {
				output[[i]] <- pred
			} else {

				for (j in 1:(steps[i]-1)) {
					# Because ε is martingale difference sequence the expected value should be zero	
					pred[j+1] <- ω + β * pred[j]
				}
				output[[i]] <- pred
			}
		}
		return(output)
	}
}

